#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
# 
# Module Name:
#   
#     TestCase.tcl
#     
# Abstract:
# 
#     Implementation for EMS misc utility function
#

if {[info exist UTILITY_TCL] == 1} {
  return
}
set UTILITY_TCL 1

# 
# Routine Description:
# 
#   Wait for the lock.
# 
# Arguments:
# 
#   None.
# 
# Returns:
# 
#   None.
#
proc EmsWaitForMutex {} {
  global _ENTS GuiBoxs GuiCursors GuiFrames

  if { $_ENTS(blockflag) != 0 } {
    return -1
  }
  
  set _ENTS(blockflag) 1

  $GuiFrames(button) configure -cursor $GuiCursors(busy)
  $GuiBoxs(output)   configure -cursor $GuiCursors(busy)
  $GuiBoxs(testcase) configure -cursor $GuiCursors(busy)
  update 
  return 0
}

# 
# Routine Description:
# 
#   Release the lock.
# 
# Arguments:
# 
#   None.
# 
# Returns:
# 
#   None.
#
proc EmsReleaseMutex {} {
  global _ENTS GuiBoxs GuiCursors GuiFrames

  $GuiFrames(button) configure -cursor $GuiCursors(default)
  $GuiBoxs(output)   configure -cursor $GuiCursors(default)
  $GuiBoxs(testcase) configure -cursor $GuiCursors(default)
  set _ENTS(blockflag) 0
}

# 
# Routine Description:
# 
#   Generate report.
# 
# Arguments:
# 
#   None.
# 
# Returns:
# 
#   None.
#
proc CmdTestReport { } {
  global _ENTS EmsActiveLeftFrame

  if {[EmsTestIsRunning] == 1} {
    tk_messageBox -message "can not generate report when test is running" -type ok 
    return
  }

  if {[EmsWaitForMutex] == -1} {
    return
  }

  if {[file exists "Report"]} {
    if {[file type "Report"] != "directory"} {
      file delete "Report" -force
      file mkdir "Report"
	}
  } else {
    file mkdir "Report"
  }

  switch $EmsActiveLeftFrame {
    "RemoteExecution"   {RemoteExecutionTestReport}
    "RemoteValidation"  {RemoteValidationTestReport}
  }
  EmsReleaseMutex
}

# 
# Routine Description:
# 
#   Show the message in the status bar.
# 
# Arguments:
# 
#   status  : The string to show
# 
# Returns:
# 
#   None.
#
proc EmsShowStatus { Status } {
  global _ENTS
  set _ENTS(status) $Status
}

# 
# Routine Description:
# 
#   The callback function of reload a case tree
# 
# Arguments:
# 
#   None
# 
# Returns:
# 
#   None.
#
proc CmdCaseTreeReload {} {
  global EmsActiveLeftFrame

  if {[EmsTestIsRunning] == 1} {
    tk_messageBox -message "can not rebuild case tree when test is running" -type ok 
    return
  }

  if {[EmsWaitForMutex] == -1} {
    return
  }

  switch $EmsActiveLeftFrame {
    "RemoteExecution"   {RemoteExecutionReload}
    "RemoteValidation"  {RemoteValidationReload}
  }
  EmsReleaseMutex
}
