#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid          62B72667-FDEF-446f-8553-A70C637BD0E7
CaseName          Configure.Conf1.Case3
CaseCategory      MTFTP4
CaseDescription   {This case is to test the EFI_INVALID_PARAMETER conformance  \
                   of Mtftp4.Configure when MtftpConfigData->ServerIp is not a \
                   valid IPv4 unicast address.}
################################################################################

#
# Begin log ...
#
BeginLog

Include MTFTP4/include/Mtftp4.inc.tcl

BeginScope _MTFTP4_CONFIGURE_CONFORMANCE1_CASE3_

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS Side Parameter"
#
UINTN                            R_Status
UINTN                            R_Handle
EFI_MTFTP4_CONFIG_DATA           R_MtftpConfigData

Mtftp4ServiceBinding->CreateChild "&@R_Handle, &@R_Status"
GetAck
SetVar     [subst $ENTS_CUR_CHILD]  @R_Handle
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mtftp4SBP.CreateChild - Create Child 1"                       \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

SetVar R_MtftpConfigData.UseDefaultSetting    FALSE
SetIpv4Address R_MtftpConfigData.StationIp    "192.168.88.88"
SetIpv4Address R_MtftpConfigData.SubnetMask   "255.255.255.0"
SetVar R_MtftpConfigData.LocalPort            0
SetIpv4Address R_MtftpConfigData.GatewayIp    "0.0.0.0"
SetIpv4Address R_MtftpConfigData.ServerIp     "255.255.255.255"
SetVar R_MtftpConfigData.InitialServerPort    69
SetVar R_MtftpConfigData.TryCount             10
SetVar R_MtftpConfigData.TimeoutValue         2

#
# check point
#
Mtftp4->Configure {&@R_MtftpConfigData, &@R_Status}
GetAck

set assert [VerifyReturnStatus R_Status $EFI_INVALID_PARAMETER]
RecordAssertion $assert $Mtftp4ConfigureConfAssertionGuid003                   \
                "Mtftp4.Configure() - returns EFI_INVALID_PARAMETER when       \
                 MtftpConfigData->UseDefaultSetting is FALSE and               \
                 MtftpConfigData->ServerIp is not a valid IPv4 unicast address."\
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_INVALID_PARAMETER"

Mtftp4ServiceBinding->DestroyChild {@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mtftp4SBP.DestroyChild - Destroy Child 1"                     \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

EndScope _MTFTP4_CONFIGURE_CONFORMANCE1_CASE3_

#
# End Log
#
EndLog