/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Copyright (c) 2006, Intel Corporation
All rights reserved. 

This UEFI SCT ("Software") is furnished under license and may only be used or copied
in accordance with the terms of that license UEFI Contributor Agreement.
No license, express or implied, by estoppel or otherwise, to any intellectual property
rights is granted by this document. The Software is subject to change without notice,
and should not be construed as a commitment by Intel Corporation to market, license,
sell or support any product or technology. Unless otherwise provided for in the license
under which this Software is provided, the Software is provided AS IS, with no warranties
of any kind, express or implied. Except as expressly permitted by the Software license,
neither Intel Corporation nor its suppliers assumes any responsibility or liability
for any errors or inaccuracies that may appear herein. 

Module Name:

  Udp4ServiceBindingENTSTest.c

Abstract:

--*/

#include "Efi.h"
#include "EfiDriverlib.h"
#include "Udp4ServiceBindingENTSTestCase.h"
#include EFI_PROTOCOL_DEFINITION (LoadedImage)

static CHAR16     gUdp4ServiceBindingProtocolName[] = L"Udp4ServiceBinding";

CHAR16            *gUdp4ServiceBindingRuntimeInfo;
UINTN             gUdp4ServiceBindingRuntimeInfoSize;

ENTS_INTERFACE    gUdp4ServiceBindingEntsInterfaceList[] = {
  {
    L"CreateChild",
    Udp4ServiceBindingCreateChild_EntsTest,
    gUdp4ServiceBindingCreateChildArgField
  },
  {
    L"DestroyChild",
    Udp4ServiceBindingDestroyChild_EntsTest,
    gUdp4ServiceBindingDestroyChildArgField
  },
  0
};

EFI_ENTS_PROTOCOL *gUdp4ServiceBindingEntsProtocolInterface = NULL;

EFI_STATUS
Udp4ServiceBindingENTSTestUnload (
  IN EFI_HANDLE                ImageHandle
  );

EFI_DRIVER_ENTRY_POINT (Udp4ServiceBindingENTSTestMain)

EFI_STATUS
Udp4ServiceBindingENTSTestMain (
  IN EFI_HANDLE                ImageHandle,
  IN EFI_SYSTEM_TABLE          *SystemTable
  )
/*++

Routine Description:

  Entry point for Udp4 service binding protocol ENTS Test.

Arguments:

  ImageHandle           - The image handle.
  SystemTable           - The system table.

Returns:

  EFI_SUCCESS - Operation succeeded.
  Others      - Some failure happened.

--*/
{
  EFI_STATUS                Status;
  EFI_LOADED_IMAGE_PROTOCOL *LoadedImage;
  EFI_HANDLE                ClientHandle;

  EfiInitializeDriverLib (ImageHandle, SystemTable);
  EfiInitializeEntsLib (ImageHandle, SystemTable);

  gBS->HandleProtocol (
        ImageHandle,
        &gEfiLoadedImageProtocolGuid,
        (VOID *) &LoadedImage
        );

  LoadedImage->Unload = Udp4ServiceBindingENTSTestUnload;

  Status = gBS->AllocatePool (
                  EfiBootServicesData,
                  sizeof (EFI_ENTS_PROTOCOL),
                  &gUdp4ServiceBindingEntsProtocolInterface
                  );
  if (EFI_ERROR (Status)) {
    goto Error;
  }

  Status = EntsNetworkServiceBindingGetControllerHandle (
             &gEfiUdp4ServiceBindingProtocolGuid, 
             &ClientHandle
             );
  if (EFI_ERROR(Status)) {
  	goto Error;
  }

  gUdp4ServiceBindingEntsProtocolInterface->ClientName        = gUdp4ServiceBindingProtocolName;
  gUdp4ServiceBindingEntsProtocolInterface->ClientAttribute   = ENTS_PROTOCOL_ATTRIBUTE_PROTOCOL;
  gUdp4ServiceBindingEntsProtocolInterface->ClientGuid        = &gEfiUdp4ServiceBindingProtocolGuid;
  gUdp4ServiceBindingEntsProtocolInterface->ClientHandle      = ClientHandle;
  gUdp4ServiceBindingEntsProtocolInterface->ClientInterface   = NULL;
  gUdp4ServiceBindingEntsProtocolInterface->EntsInterfaceList = gUdp4ServiceBindingEntsInterfaceList;
  gUdp4ServiceBindingEntsProtocolInterface->RuntimeInfo       = gUdp4ServiceBindingRuntimeInfo;
  gUdp4ServiceBindingEntsProtocolInterface->RuntimeInfoSize   = gUdp4ServiceBindingRuntimeInfoSize;

  Status = gBS->InstallMultipleProtocolInterfaces (
                  &ImageHandle,
                  &gEfiEntsProtocolGuid,
                  gUdp4ServiceBindingEntsProtocolInterface,
                  NULL
                  );
  if (EFI_ERROR (Status)) {
    goto Error;
  }

  return EFI_SUCCESS;

Error:
  if (gUdp4ServiceBindingEntsProtocolInterface != NULL) {
    gBS->FreePool (gUdp4ServiceBindingEntsProtocolInterface);
  }

  return Status;
}

EFI_STATUS
Udp4ServiceBindingENTSTestUnload (
  IN EFI_HANDLE                ImageHandle
  )
/*++

Routine Description:

  Unload routine.

Arguments:

  ImageHandle - The image handle.

Returns:

  EFI_SUCCESS - Operation succeeded.
  Others      - Some failure happened.

--*/
{
  EFI_STATUS  Status;

  Status = gBS->UninstallMultipleProtocolInterfaces (
                  ImageHandle,
                  &gEfiEntsProtocolGuid,
                  gUdp4ServiceBindingEntsProtocolInterface,
                  NULL
                  );

  if (gUdp4ServiceBindingEntsProtocolInterface != NULL) {
    gBS->FreePool (gUdp4ServiceBindingEntsProtocolInterface);
  }

  return Status;
}
