/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  EntsTime.c

Abstract:

  ENTS time services implementations.

--*/

#include "EntsLib.h"

//
// Global variables and definitions
//
#define IS_LEAP(y)  (((y) % 4) == 0 && ((y) % 100) != 0 || ((y) % 400) == 0)

static const UINTN  MonthLengths[2][12] = {
  {
    31,
    28,
    31,
    30,
    31,
    30,
    31,
    31,
    30,
    31,
    30,
    31
  },
  {
    31,
    29,
    31,
    30,
    31,
    30,
    31,
    31,
    30,
    31,
    30,
    31
  }
};

UINT32
SecondsElapsedFromBaseYear (
  IN UINT16             BaseYear,
  IN UINT16             Year,
  IN UINT8              Month,
  IN UINT8              Day,
  IN UINT8              Hour,
  IN UINT8              Minute,
  IN UINT8              Second
  )
/*++

Routine Description:

  Calculate the elapsed seconds from the base year.
  
Arguments:

  BaseYear  - The base year.
  Year      - Current year.
  Month     - Current month.
  Day       - Current day.
  Hour      - Current hour.
  Minute    - Current minute.
  Second    - Current second.

Returns:

  The elapsed seconds.

--*/
{
  UINTN   Seconds;
  UINT32  LeapYear;
  INTN    Index;

  Seconds = 0;
  for (Index = BaseYear; Index < Year; Index++) {
    if (IS_LEAP (Index)) {
      Seconds += DAYS_PER_LYEAR * SECS_PER_DAY;
    } else {
      Seconds += DAYS_PER_NYEAR * SECS_PER_DAY;
    }
  }

  LeapYear = IS_LEAP (Year);
  for (Index = 0; Index < Month - 1; Index++) {
    Seconds += MonthLengths[LeapYear][Index] * SECS_PER_DAY;
  }

  for (Index = 0; Index < Day - 1; Index++) {
    Seconds += SECS_PER_DAY;
  }

  for (Index = 0; Index < Hour; Index++) {
    Seconds += SECS_PER_HOUR;
  }

  for (Index = 0; Index < Minute; Index++) {
    Seconds += SECS_PER_MIN;
  }

  return (UINT32) (Seconds + Second);
}
