/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
 
  EftpWRQ.h
 
Abstract:

  WRQ procedure
 
--*/

#ifndef _EFI_EFTP_WRQ_H_
#define _EFI_EFTP_WRQ_H_

//
// Init a silent shutdown.
// A silent shutdown will discard any incoming packet, and send nothing
//
#define WRQ_SILENT_SHUTDOWN(Private) \
  do { \
    (Private)->State = EFTP_TIME_WAIT; \
    EftpSetTimer ((Private), 0, EFTP_TIMEWAIT_TIME); \
  } while (0)

  //
  // Init a loud shutdown
  // A loud shutdown will discard any incoming packet, and send out last control packet
  //
#define WRQ_LOUD_SHUTDOWN(Private) \
  do { \
    ASSERT ((Private)->State != EFTP_TIME_WAIT); \
    (Private)->State = EFTP_WRQ_CLOSEING; \
    EftpSetTimer ((Private), EFTP_CLOSEING_TIME, 0); \
  } while (0)


//
// Vars specific to WRQ
//
typedef struct _EFTP_WRQ_STATE {
  EFTP_PACKET_BUFFER  *LastData;  // saved for retransmission
} EFTP_WRQ_STATE;

EFTP_WRQ_STATE  *
EftpInitWrqState (
  IN EFTP_IO_PRIVATE*Private
  )
/*++

Routine Description:

  Init a Eftp instance for WRQ, or uploading

Arguments:

  Private - Eftp instance private data

Returns:

  NULL: Failed to allocate buffer for WRQ state
  Point to the newly allocated WRQ data structure: On success

--*/
;

VOID
EftpWrqRxCallback (
  IN EFI_EVENT  Event,
  IN VOID       *Context
  )
/*++

Routine Description:

  Callback function by UDP when it receives packets for the instance.
  It will dispatch the packet to different routines for process.

Arguments:

  Event   : Event signalled by UDP
  Context : Event's context, it is a point to EFTP_COMPLETION_TOKEN

Returns:

  None 

--*/
;

EFI_STATUS
EftpBuildWrq (
  IN EFTP_IO_PRIVATE*Private
  )
/*++

Routine Description:

  Build a WRQ packet from the user info, and save it at Private->Request

Arguments:

  Private - Eftp instance private data

Returns:

  EFI_OUT_OF_RESOURCES: Failed to allocate buffer for WRQ packet
  EFI_SUCCESS : WRQ packet allocated and initialized

--*/
;

VOID
EftpWrqTimer (
  IN EFI_EVENT  Event,
  IN VOID       *Context
  )
/*++

Routine Description:

  Timer function when timeout happened during uploading.
  It will either retransmit the last packet, transit the state,
  or clean up the instance
  
Arguments:

  Event   : Timer event got signaled
  Context : The event's context. It is a point to EFTP_IO_PRIVATE

Returns:

  Event callback returns nothing

--*/
;

VOID
EftpWrqCleanUp (
  IN EFTP_IO_PRIVATE*private
  )
/*++

Routine Description:

  Gracefully shutdown the Eftp instance: release various resources
  and signal the user event

Arguments:

  Private : Eftp instance private data

Returns:

  None

--*/
;

#endif
