/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  Dialog.h

Abstract:

  This file defines Dialog data structure.

--*/

#ifndef _EFI_DIALOG_H_
#define _EFI_DIALOG_H_

//
// Includes
//
#include "Efi.h"
#include "Ui.h"
#include EFI_PROTOCOL_DEFINITION(DevicePath)

//
// Definitions
//

typedef enum {
  EFI_DIALOG_TYPE_YES_OR_NO = 0,
  EFI_DIALOG_TYPE_MESSAGE,
  EFI_DIALOG_TYPE_REMINDER,
  EFI_DIALOG_TYPE_MAX
} EFI_DIALOG_TYPE;

#define DIALOG_MENU_AREA       0
#define DIALOG_EDIT_AREA       1

//
//File Dialog Definition
//
typedef struct  {
  EFI_DIALOG_TYPE               Type;
  CHAR16                        *Title;
  EFI_RECT                      BodyRect;
  UINT32                        BackColor;
  UINT32                        ForeColor;
} EFI_DIALOG;

//
//file dialog context
//
typedef struct {
  EFI_DIALOG_TYPE               Type;
  UINT32                        ChooseNumber;
} EFI_DIALOG_CONTEXT;

//
// Functions
//

EFI_STATUS
DoDialog(
  IN CHAR16                        *DialogTitle,
  IN OUT EFI_DIALOG_CONTEXT        *DialogContext
  )
/*++

Routine Description:

  Through file dialog open or save a file.

Arguments:

  Context     - The EFI_FILE_DIALOG_CONTEXT instance pointer.

Returns:

  EFI_SUCCESS           - the FileName and DevicePath was set successfully.
  EFI_INVALID_PARAMETER - the Parameter is Invalid

--*/
;

#endif
